﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Web;
using System.Web.Caching;
using System.Runtime.Serialization.Formatters.Binary;

public class DiskCacheProvider :OutputCacheProvider
{

    [Serializable]
    private class DiskCacheEntry
    {
        public DateTime Expire { get; set; }
        public object Entry { get; set; }
    }

    private string baseCachePath = "~/App_Data/DiskCache/";

	public DiskCacheProvider()
	{
	}

    public override object Add(string key, object entry, DateTime utcExpiry)
    {
        string filename = HttpContext.Current.Server.MapPath(baseCachePath + key.GetHashCode() + ".cache");

        using (System.IO.Stream ms = File.OpenWrite(filename))
        {
            DiskCacheEntry cacheItem = new DiskCacheEntry{Expire = utcExpiry, Entry = entry};

            BinaryFormatter formatter = new BinaryFormatter();
            formatter.Serialize(ms, cacheItem);
            ms.Flush();
            ms.Close();
            formatter = null;
        }

        return entry;
    }

    public override object Get(string key)
    {
        string filename = HttpContext.Current.Server.MapPath(baseCachePath + key.GetHashCode() + ".cache");
        if (!File.Exists(filename))
            return null;

        bool markForDelete = false;
        using (FileStream fs = File.Open(filename, FileMode.Open))
        {
            BinaryFormatter formatter = new BinaryFormatter();
            object obj = formatter.Deserialize(fs);
            formatter = null;

            DiskCacheEntry cacheItem = obj as DiskCacheEntry;
            if (obj != null)
            {
                markForDelete = cacheItem.Expire <= DateTime.Now.ToUniversalTime();
            }

            if (!markForDelete)
                return cacheItem.Entry;
        }

        if (markForDelete)
            File.Delete(filename);
        
        return null;
    }

    public override void Remove(string key)
    {
        string filename = HttpContext.Current.Server.MapPath(baseCachePath + key.GetHashCode() + ".cache");
        File.Delete(filename);
    }

    public override void Set(string key, object entry, DateTime utcExpiry)
    {
        Add(key, entry, utcExpiry);
    }
}